[Event ("onMotionChanged")]
[Event ("onMotionFinished")]
[Event ("onMotionLooped")]
[Event ("onMotionResumed")]
[Event ("onMotionStarted")]
[Event ("onMotionStopped")]

/**
* A MotionCamera instance can record the movement of a single "actor" property onto
* its own "film". It can then play it back, to the same property (the actor)
* or to a different property.
* 
* @author Robert Penner (AS 1.0)
*  2002 Robert Penner
* http://www.robertpenner.com
* 
* @author Mark Walters (AS 2.0)
*  2006 DigitalFlipbook
* http://www.digitalflipbook.com
* 
* @version	1.0
* 
* @since	ActionScript 2.0; Flash Player 6
*/
class com.robertpenner.effects.MotionCamera extends mx.transitions.Tween {
	
	/*
	* CLASS PROPERTIES
	*/
	private var $actor:Object;
	private var $actorProp:String;
	private var $film:Array;
	private var isRecording:Boolean = false;
	
	/*
	* EVENTS
	*/
	
	/**
	* Event broadcast with each change in the tweened object's property that is being animated.
	* 
	* @usage 	<pre><code>
	* myTween.onMotionChanged = function() {
    * 	// ...
	* };
	* </code></pre>
	*/
	public var onMotionChanged:Function;
	
	/**
	* Event broadcast when the Tween object finishes its animation.
	* 
	* @usage 	<pre><code>
	* myTween.onMotionFinished = function() {
    * 	// ...
	* };
	* </code></pre>
	*/
	public var onMotionFinished:Function;
	
	/**
	* Event broadcast when the Tween object loops.
	* 
	* @usage 	<pre><code>
	* myTween.onMotionLooped = function() {
    * 	// ...
	* };
	* </code></pre>
	*/
	public var onMotionLooped:Function;
	
	/**
	* Event broadcast when the Tween.resume() method is called, causing the tweened animation to resume.
	* 
	* @usage 	<pre><code>
	* myTween.onMotionResumed = function() {
    * 	// ...
	* };
	* </code></pre>
	*/
	public var onMotionResumed:Function;
	
	/**
	* Event broadcast when the Tween.start() method is called, causing the tweened animation to start.
	* 
	* @usage 	<pre><code>
	* myTween.onMotionStarted = function() {
    * 	// ...
	* };
	* </code></pre>
	*/
	public var onMotionStarted:Function;
	
	/**
	* Event broadcast when the Tween.stop() method is called, causing the tweened animation to stop.
	* 
	* @usage 	<pre><code>
	* myTween.onMotionStopped = function() {
    * 	// ...
	* };
	* </code></pre>
	*/
	public var onMotionStopped:Function;
	
	
	/*
	* *********************************************************
	* CONSTRUCTOR
	* *********************************************************
	*
	*/
	
	/**
	* Creates an instance of the MotionCamera class.
	* 
	* @param	obj			The object which the MotionCamera targets.
	* @param	prop		The name of the property (in obj) that will be affected.
	* @param	duration	The length of time of the motion; set to infinity if negative, 0 or omitted.
	* 
	* @usage
	* <pre><code>
	* var xCam:MotionCamera = new MotionCamera(this, "_x", 0);
	* </code></pre>
	*/
	function MotionCamera (obj:Object, prop:String, duration:Number) {
		super (obj, prop, null, obj[prop], null, duration);
		this.setActor (obj, prop);
		this.film = [];
	}
	
	
	/*
	* *********************************************************
	* PRIVATE METHODS
	* *********************************************************
	* 
	*/
	
	private function update ():Void {
		if (this.isRecording) 
			with (this) film[time] = actor[actorProp];
		super.update();
	}
	
	
	/*
	* *********************************************************
	* PUBLIC METHODS
	* *********************************************************
	* 
	*/
	
	/**
	* Turns the MotionCamera's recording mode on.
	* 
	* @usage
	* <pre><code>
	* xCam.startRecord();
	* </code></pre> 
	*/
	public function startRecord ():Void {
		this.isRecording = true;
	}
	
	/**
	* Turns the MotionCamera's recording mode off.
	* 
	* @usage
	* <pre><code>
	* xCam.stopRecord();
	* </code></pre> 
	*/
	public function stopRecord ():Void {
		this.isRecording = false;
	}
	
	/**
	* Cuts the film to a specific length.
	* 
	* @param	t		A specific point in time to cut the film; uses the current time if left blank.
	* 
	* @usage
	* <pre><code>
	* xCam.cutFilm();
	* </code></pre> 
	*/
	public function cutFilm (t:Number):Void {
		if (t == undefined) var t:Number = this.time;
		this.film.length = t + 1;
		this.duration = t;
	}
	
	/**
	* Blanks out the film. Effectively clears the data.
	* 
	* @usage
	* <pre><code>
	* xCam.eraseFilm();
	* </code></pre> 
	*/
	public function eraseFilm ():Void {
		this.film = [];
		this.duration = 0;
		this.time = 0;
	}
	
	/**
	* Returns the position at a specified time.
	* 
	* @param	t	The time to check the object's position; set to the current time if omitted.
	* @return	The position.
	* 
	* @usage
	* <pre><code>
	* trace (xCam.getPosition());
	* </code></pre>
	*/
	public function getPosition (t:Number):Number {
		if (t == undefined) t = this.time;
		return this.film[t];
	}
	
	/**
	* Sets the actor and actor's property to be targetted by the MotionCamera.
	* 
	* @param	actor		The actor to be targetted by the MotionCamera.
	* @param	actorProp	The actor's property to be targetted by the MotionCamera.
	* 
	* @usage
	* <pre><code>
	* trace (xCam.setActor(this, "_y"));
	* </code></pre>
	*/
	public function setActor (actor:Object, actorProp:String):Void {
		this.actor = actor;
		if (actorProp) this.actorProp = actorProp;
	}
	
	/**
	* Instructs the tweened animation to continue from its current value to a new value.
	* 
	* @param	finish		A number indicating the ending value of the target object property that is to be tweened.
	* @param	duration	A number indicating the length of time or number of frames for the tween motion.
	* 
	* @usage
	* <pre><code>
	* myTween.continueTo(myFinish, myDuration);
	* </code></pre>
	*/
	public function continueTo (finish:Number, duration:Number):Void {
		super.continueTo(finish, duration);
	}
	
	/**
	* Forwards the tweened animation directly to the end of the animation.
	* 
	* @usage
	* <pre><code>
	* myTween.fforward();
	* </code></pre>
	*/
	public function fforward ():Void {
		super.fforward();
	}
	
	/**
	* Forwards the tweened animation to the next frame.
	* 
	* @usage
	* <pre><code>
	* myTween.nextFrame();
	* </code></pre>
	*/
	public function nextFrame ():Void {
		super.nextFrame();
	}
	
	/**
	* Directs the tweened animation to the frame previous to the current frame.
	* 
	* @usage
	* <pre><code>
	* myTween.prevFrame();
	* </code></pre>
	*/
	public function prevFrame ():Void {
		super.prevFrame();
	}
	
	/**
	* Resumes a tweened animation from its stopped point in the animation.
	* 
	* @usage
	* <pre><code>
	* myTween.resume();
	* </code></pre>
	*/
	public function resume ():Void {
		super.resume();
	}
	
	/**
	* Rewinds a tweened animation to the beginning of the tweened animation.
	* 
	* @usage
	* <pre><code>
	* myTween.rewind();
	* </code></pre>
	*/
	public function rewind ():Void {
		super.rewind();
	}
	
	/**
	* Starts the tweened animation from the beginning.
	* 
	* @usage
	* <pre><code>
	* myTween.start();
	* </code></pre>
	*/
	public function start ():Void {
		super.start();
	}
	
	/**
	* Stops the tweened animation at its current position.
	* 
	* @usage
	* <pre><code>
	* myTween.stop();
	* </code></pre>
	*/
	public function stop ():Void {
		super.stop();
	}
	
	/**
	* Returns a summarization of the MotionCamera instance.
	* 
	* @return	Returns a summarization of the MotionCamera instance.
	* 
	* @usage
	* <pre><code>
	* trace (xCam.toString());
	* trace (xCam);
	* </code></pre>
	*/
	public function toString ():String {
		return "[MotionCamera prop=" + this.prop + " t=" + this.time + " pos=" + this.position + " mode: " + this.mode + "]";
	}
	
	/**
	* Traces a comma-delimited list of the film to the Output window.
	* 
	* @usage
	* <pre><code>
	* xCam.print();
	* </code></pre> 
	*/
	public function print ():Void {
		trace (this.filmString);
	}
	
	/**
	* Instructs the tweened animation to play in reverse from its last direction of tweened property increments.
	* 
	* @usage
	* <pre><code>
	* myTween.yoyo();
	* </code></pre>
	*/
	public function yoyo ():Void {
		super.yoyo();
	}

	
	/*
	* *********************************************************
	* GETTERS/SETTERS
	* *********************************************************
	* 
	*/
	
	/**
	* The actor to be targetted by the MotionCamera.
	* 
	* @usage
	* <pre><code>
	* xCam.actor = this;
	* </code></pre>
	*/
	public function get actor ():Object {
		return this.$actor;
	}
	public function set actor (a:Object):Void {
		this.$actor = a;
	}
	
	/**
	* The actor's property to be targetted by the MotionCamera.
	* 
	* @usage
	* <pre><code>
	* xCam.actorProp = "_y";
	* </code></pre>
	*/
	public function get actorProp ():String {
		return this.$actorProp;
	}
	public function set actorProp (ap:String):Void {
		this.$actorProp = ap;
	}
	
	/**
	* The film of the MotionCamera.
	* 
	* @usage
	* <pre><code>
	* xCam.film = [24, 53, 62, 59, 86, 23, 123];
	* </code></pre>
	*/
	public function get film ():Array {
		return this.$film;
	}
	public function set film (f:Array):Void {
		this.$film = f;
	}
	
	/**
	* The film of the MotionCamera as a comma-delimited list.
	* 
	* @usage
	* <pre><code>
	* xCam.filmString = "24, 53, 62, 59, 86, 23, 123";
	* </code></pre>
	*/
	public function get filmString ():String {
		return this.$film.toString();
	}
	public function set filmString (f:String):Void {
		this.$film = f.split (",");
	}
	
	/**
	* The current mode of the MotionCamera.
	* 
	* @usage
	* <pre><code>
	* trace (xCam.mode);
	* </code></pre>
	*/
	public function get mode ():String {
		if (this.isRecording) return "recording"
		else if (this.isPlaying) return "playing"
		else return "off";
	}
	
	/**
	* The duration of the tweened animation in frames or seconds.
	* 
	* @usage
	* <pre><code>
	* trace (myTween.duration);
	* </code></pre>
	*/
	public function get duration ():Number {
		return super.duration;
	}
	public function set duration (d:Number):Void {
		super.duration = d;
	}
	
	/**
	* The last tweened value for the end of the tweened animation.
	* 
	* @usage
	* <pre><code>
	* trace (myTween.finish);
	* </code></pre>
	*/
	public function get finish ():Number {
		return super.finish;
	}
	public function set finish (f:Number):Void {
		super.finish = f;
	}
	
	/**
	* The number of frames per second of the tweened animation.
	* 
	* @usage
	* <pre><code>
	* trace (myTween.FPS);
	* </code></pre>
	*/
	public function get FPS ():Number {
		return super.FPS;
	}
	public function set FPS (fps:Number):Void {
		super.FPS = fps;
	}
	
	/**
	* The current value of the target movie clip's property being tweened.
	* 
	* @usage
	* <pre><code>
	* trace (myTween.position);
	* </code></pre>
	*/
	public function get position ():Number {
		return super.position;
	}
	public function set position (p:Number):Void {
		super.position = p;
	}
	
	/**
	* The current time within the duration of the animation.
	* 
	* @usage
	* <pre><code>
	* trace (myTween.time);
	* </code></pre>
	*/
	public function get time ():Number {
		return super.time;
	}
	public function set time (t:Number):Void {
		super.time = t;
	}
}